@@ -1,4 +1,6 @@
-__version__ = '3.3.0'
+__version__ = '4.2.0'
+
+from functools import wraps
 
 from tri_declarative import LAST
 
@@ -59,12 +61,30 @@ def render_if_needed(request, response):
 
 
 def middleware(get_response):
+    from django.db import connections
+    atomic_request_connections = [db for db in connections.all() if db.settings_dict['ATOMIC_REQUESTS']]
+    if any(atomic_request_connections):
+        raise TypeError(
+            'The iommi middleware is unable to retain atomic transactions. Disable ATOMIC_REQUEST for '
+            f'database connections ({", ".join(db.settings_dict["NAME"] for db in atomic_request_connections)}) '
+            f'or remove middleware and use the @iommi_render decorator on the views instead.'
+        )
+
     def iommi_middleware(request):
         return render_if_needed(request, get_response(request))
 
     return iommi_middleware
 
 
+def iommi_render(view):
+    @wraps(view)
+    def inner(request, *args, **kwargs):
+        result = view(request, *args, **kwargs)
+        return render_if_needed(request, result)
+
+    return inner
+
+
 __all__ = [
     'Action',
     'Asset',
@@ -93,6 +113,7 @@ __all__ = [
     'register_search_fields',
     'Style',
     'html',
+    'iommi_render',
 ]
 
 default_app_config = 'iommi.apps.IommiConfig'
