@@ -31,10 +31,7 @@ from iommi.fragment import (
 )
 from iommi.member import Members
 from iommi.part import Part
-from iommi.reinvokable import reinvokable
-from iommi.traversable import (
-    EvaluatedRefinable,
-)
+from iommi.refinable import EvaluatedRefinable
 
 
 @with_meta
@@ -101,20 +98,20 @@ class Action(Fragment):
 
     @dispatch(
         tag='a',
-        attrs=EMPTY,
-        children=EMPTY,
         display_name=lambda action, **_: capitalize(action._name).replace('_', ' '),
     )
-    @reinvokable
-    def __init__(self, *, tag=None, attrs=None, children=None, display_name=None, **kwargs):
-        if tag == 'input':
-            if display_name and 'value' not in attrs:
-                attrs.value = display_name
+    def __init__(self, **kwargs):
+        super().__init__(**kwargs)
+
+    def on_refine_done(self):
+        if self.tag == 'input':
+            if self.display_name and 'value' not in self.attrs:
+                self.attrs.value = self.display_name
         else:
-            children['text'] = display_name
-            if tag == 'button' and 'value' in attrs:
+            self.children['text'] = self.display_name
+            if self.tag == 'button' and 'value' in self.attrs:
                 assert False, 'You passed attrs__value, but you should pass display_name'
-        super().__init__(tag=tag, attrs=attrs, children=children, display_name=display_name, **kwargs)
+        super().on_refine_done()
 
     def on_bind(self):
         super().on_bind()
@@ -197,11 +194,3 @@ def group_actions(actions: Dict[str, Action]):
 class Actions(Members, Tag):
     attrs: Attrs = Refinable()  # attrs is evaluated, but in a special way so gets no EvaluatedRefinable type
     tag = EvaluatedRefinable()
-
-    @dispatch(
-        attrs__class=EMPTY,
-        attrs__style=EMPTY,
-    )
-    @reinvokable
-    def __init__(self, **kwargs):
-        super(Actions, self).__init__(**kwargs)
