@@ -36,6 +36,9 @@ def model_and_rows(model, rows):
 
 
 def build_as_view_wrapper(target):
+    if not target.is_refine_done:
+        target = target.refine_done()
+
     def view_wrapper(request, **url_params):
         request.url_params = url_params
         return target.bind(request=request).render_to_response()
@@ -72,4 +75,4 @@ def get_display_name(part):
             return capitalize(part.model_field.verbose_name)
     except AttributeError:
         pass
-    return capitalize(force_str(part._name).rsplit('__', 1)[-1].replace("_", " "))
+    return capitalize(force_str(part._name).replace("_", " "))
