@@ -36,7 +36,6 @@ from tri_declarative import (
     declarative,
     dispatch,
     EMPTY,
-    flatten,
     getattr_path,
     Namespace,
     Refinable,
@@ -64,7 +63,6 @@ from iommi.base import (
     NOT_BOUND_MESSAGE,
     values,
 )
-from iommi.debug import iommi_debug_on
 from iommi.endpoint import path_join
 from iommi.evaluate import evaluate
 from iommi.form import (
@@ -86,21 +84,15 @@ from iommi.from_model import (
 )
 from iommi.member import (
     bind_members,
-    collect_members,
+    refine_done_members,
 )
 from iommi.part import (
     Part,
     request_data,
 )
-from iommi.reinvokable import (
-    reinvokable,
-    reinvoke,
-    set_and_remember_for_reinvoke,
-)
-from iommi.traversable import (
-    declared_members,
+from iommi.refinable import (
     EvaluatedRefinable,
-    set_declared_member,
+    RefinableMembers,
 )
 
 
@@ -240,16 +232,12 @@ class Filter(Part):
     is_valid_filter = Refinable()
     query_name = EvaluatedRefinable()
 
-    @reinvokable
     @dispatch(
         query_operator_for_field='=',
         attr=MISSING,
         search_fields=MISSING,
         field__required=False,
-        # TODO: this isn't right, freetext can be a callable
-        field__include=lambda query, field, **_: not query.filters._declared_members.get(
-            field._name, Struct(freetext=False)
-        ).freetext,
+
         is_valid_filter=default_filter__is_valid_filter,
         query_name=lambda filter, **_: filter.iommi_name(),
     )
@@ -260,12 +248,14 @@ class Filter(Part):
         :param field__include: set to `True` to display a GUI element for this filter in the basic style interface.
         :param field__call_target: the factory to create a `Field` for the basic GUI, for example `Field.choice`. Default: `Field`
         """
-        model_field = kwargs.get('model_field')
-        if model_field and model_field.remote_field:
-            kwargs['model'] = model_field.remote_field.model
-
         super(Filter, self).__init__(**kwargs)
 
+    def on_refine_done(self):
+        model_field = self.model_field
+        if model_field and model_field.remote_field:
+            self.model = model_field.remote_field.model
+        super(Filter, self).on_refine_done()
+
     def on_bind(self) -> None:
         if self.attr is MISSING:
             self.attr = self._name
@@ -559,29 +549,28 @@ class QueryAutoConfig(AutoConfig):
 class Advanced(Fragment):
     toggle: Namespace = Refinable()
 
-    @dispatch(toggle=EMPTY)
-    @reinvokable
+    @dispatch(
+        toggle__call_target=Action,
+        toggle__attrs__href='#',
+        toggle__attrs__class__iommi_query_toggle_simple_mode=True,
+        toggle__attrs={'data-advanced-mode': 'simple'},
+    )
     def __init__(self, **kwargs):
         super(Advanced, self).__init__(**kwargs)
 
-        toggle = setdefaults_path(
-            Namespace(),
-            self.toggle,
+    def on_refine_done(self):
+        self.toggle = self.toggle(
             _name='toggle',
-            call_target=Action,
-            attrs__href='#',
-            attrs__class__iommi_query_toggle_simple_mode=True,
-            attrs={'data-advanced-mode': 'simple'},
             display_name=gettext('Switch to advanced search'),
-        )
-        self.toggle = declared_members(self).toggle = toggle()
+        ).refine_done(parent=self)
+        super(Advanced, self).on_refine_done()
 
     def on_bind(self) -> None:
         super(Advanced, self).on_bind()
-        self.toggle = self._bound_members.toggle = self.toggle.bind(parent=self)
+        self.toggle = self.toggle.bind(parent=self)
 
 
-@declarative(Filter, '_filters_dict')
+@declarative(Filter, '_filters_dict', add_init_kwargs=False)
 @with_meta
 class Query(Part):
     """
@@ -598,6 +587,7 @@ class Query(Part):
         )
     """
 
+    auto: Namespace = Refinable()
     form: Namespace = Refinable()
     advanced: Namespace = Refinable()
     model: Type[Model] = Refinable()  # model is evaluated, but in a special way so gets no EvaluatedRefinable type
@@ -608,11 +598,13 @@ class Query(Part):
     member_class = Refinable()
     form_class = Refinable()
 
+    # Filters need to be at the end to not steal the short names
+    filters: Namespace = RefinableMembers()
+
     class Meta:
         member_class = Filter
         form_class = Form
 
-    @reinvokable
     @dispatch(
         endpoints__errors__func=default_endpoint__errors,
         filters=EMPTY,
@@ -623,101 +615,53 @@ class Query(Part):
         form_container__attrs__class__iommi_query_form_simple=True,
         advanced__call_target=Advanced,
     )
-    def __init__(self, *, model=None, rows=None, filters=None, _filters_dict=None, auto=None, **kwargs):
-        assert isinstance(filters, dict)
+    def __init__(self, **kwargs):
+        super(Query, self).__init__(**kwargs)
 
-        if auto:
-            auto = QueryAutoConfig(**auto)
-            auto_model, auto_rows, filters = self._from_model(
+    def on_refine_done(self):
+        assert isinstance(self.filters, dict)
+
+        if self.auto:
+            auto = QueryAutoConfig(**self.auto).refine_done(parent=self)
+            model, rows, filters = self._from_model(
                 model=auto.model,
                 rows=auto.rows,
-                filters=filters,
                 include=auto.include,
                 exclude=auto.exclude,
             )
 
-            assert model is None, (
+            assert self.model is None, (
                 "You can't use the auto feature and explicitly pass model. "
                 "Either pass auto__model, or we will set the model for you from auto__rows"
             )
-            model = auto_model
 
-            if rows is None:
-                rows = auto_rows
+            if self.model is None:
+                self.model = model
 
-        model, rows = model_and_rows(model, rows)
+            if self.rows is None:
+                self.rows = rows
 
-        setdefaults_path(
-            kwargs,
-            form__call_target=self.get_meta().form_class,
-        )
+            self.filters.update(filters)
+
+        self.model, self.rows = model_and_rows(self.model, self.rows)
 
         self.query_advanced_value = None
         self.query_error = None
 
-        # Here we need to remove the freetext config from kwargs because we want to
-        # handle it differently from the other fields.
-        # BUT It's not safe to modify kwargs deeply! Because reinvoke() is evil:
-        # It will call init again with the same kwargs + whatever additional kwargs
-        # might have come from a parent or styling info.
-        freetext_config = kwargs.get('form', {}).get('fields', {}).get('freetext', {})
-        if 'form' in kwargs and 'fields' in kwargs['form'] and 'freetext' in kwargs['form']['fields']:
-            # copy (just) the namespace so we can safely remove freetext from it
-            kwargs = Namespace(flatten(Namespace(kwargs)))
-            freetext_config = kwargs.get('form', {}).get('fields', {}).pop('freetext', {})
+        refine_done_members(self, name='filters', members_from_namespace=self.filters, members_from_declared=self.get_declared('_filters_dict'), cls=self.get_meta().member_class)
 
-        super(Query, self).__init__(model=model, rows=rows, **kwargs)
+        self._on_refine_done_form()
 
-        collect_members(self, name='filters', items=filters, items_dict=_filters_dict, cls=self.get_meta().member_class)
+        self.advanced = self.advanced(_name='advanced').refine_done(parent=self)
+        self.form_container = self.form_container(_name='form_container').refine_done(parent=self)
 
-        field_class = self.get_meta().form_class.get_meta().member_class
-
-        declared_fields = Struct()
-        declared_fields[FREETEXT_SEARCH_NAME] = field_class(
-            _name=FREETEXT_SEARCH_NAME,
-            display_name=gettext('Search'),
-            required=False,
-            include=False,
-            help__include=False,
-            **freetext_config,
-        )
-
-        for name, filter in items(declared_members(self).filters):
-            field = setdefaults_path(
-                Namespace(),
-                filter.field,
-                _name=name,
-                model_field=filter.model_field,
-                attr=name if filter.attr is MISSING else filter.attr,
-                call_target__cls=field_class,
-                help__include=False,
-            )
-            declared_fields[name] = field()
-
-        # noinspection PyCallingNonCallable
-        self.form: Form = self.form(
-            _name='form',
-            _fields_dict=declared_fields,
-            attrs__method='get',
-            actions__submit=dict(
-                attrs={'data-iommi-filter-button': ''},
-                display_name=gettext('Filter'),
-            ),
-        )
-        declared_members(self).form = self.form
-
-        self.advanced = declared_members(self).advanced = self.advanced(_name='advanced')
-
-        self.form_container = self.form_container(_name='form_container')
-
-        # Filters need to be at the end to not steal the short names
-        set_declared_member(self, 'filters', declared_members(self).pop('filters'))
+        super(Query, self).on_refine_done()
 
     @dispatch(
         render__call_target=render_template,
     )
     def __html__(self, *, render=None):
-        if not self.iommi_bound_members().filters._bound_members:
+        if not self.iommi_bound_members().filters.iommi_bound_members():
             return ''
 
         setdefaults_path(
@@ -728,6 +672,57 @@ class Query(Part):
 
         return render(request=self.get_request())
 
+    def _on_refine_done_form(self):
+
+        field_class = self.get_meta().form_class.get_meta().member_class
+
+        declared_fields = Struct()
+        declared_fields[FREETEXT_SEARCH_NAME] = field_class(
+            _name=FREETEXT_SEARCH_NAME,
+            display_name=gettext('Search'),
+            required=False,
+            include=any(filter.freetext for filter in values(self.iommi_namespace.filters)),
+            help__include=False,
+            **self.iommi_namespace.form.get('fields', {}).get(FREETEXT_SEARCH_NAME, {}),
+        )
+
+        for name, filter in items(self.iommi_namespace.filters):
+            declared_fields[name] = setdefaults_path(
+                Namespace(),
+                filter.field,
+                _name=name,
+                call_target__cls=field_class,
+                model_field=filter.model_field,
+                attr=name if filter.attr is MISSING else filter.attr,
+                help__include=False,
+                # TODO: this isn't right, freetext can be a callable
+                include=filter.include and not filter.freetext,
+            )
+
+        # Remove fields from the form that correspond to non-included filters
+        declared_filters = self.iommi_namespace.filters
+        for name, field in items(declared_fields):
+            if name == FREETEXT_SEARCH_NAME:
+                continue
+            # We need to check if it's in declared_filters first, otherwise we remove any injected fields
+            if name in declared_filters and name not in self.iommi_namespace.filters:
+                declared_fields[name] = field.refine(include=False)
+
+        form_args = self.form
+
+        # noinspection PyCallingNonCallable
+        self.form: Form = self.get_meta().form_class(**setdefaults_path(
+            Namespace(),
+            form_args,
+            _name='form',
+            fields=declared_fields,
+            attrs__method='get',
+            actions__submit=dict(
+                attrs={'data-iommi-filter-button': ''},
+                display_name=gettext('Filter'),
+            ),
+        )).refine_done(parent=self)
+
     def on_bind(self) -> None:
         # Prevent the nested form from thinking it's a part of a nested form set up
         if 'form' in self._evaluate_parameters:
@@ -738,34 +733,6 @@ class Query(Part):
         request = self.get_request()
         self.query_advanced_value = request_data(request).get(self.get_advanced_query_param(), '') if request else ''
 
-        # TODO: should it be possible to have freetext as a callable? this code just treats callables as truthy
-        if any(f.freetext for f in values(declared_members(self)['filters'])):
-            set_and_remember_for_reinvoke(declared_members(self.form).fields[FREETEXT_SEARCH_NAME], include=True)
-
-        declared_fields = declared_members(self.form)['fields']
-        for name, filter in items(self.filters):
-            is_valid, message = filter.is_valid_filter(name=name, filter=filter)
-            assert is_valid, message
-            if name in declared_fields:
-                field = setdefaults_path(
-                    Namespace(),
-                    _name=name,
-                    attr=name if filter.attr is MISSING else filter.attr,
-                    model_field=filter.model_field,
-                    help__include=False,
-                )
-                declared_fields[name] = reinvoke(declared_fields[name], field)
-        set_declared_member(self.form, 'fields', declared_fields)
-
-        # Remove fields from the form that correspond to non-included filters
-        declared_filters = declared_members(self)['filters']
-        for name, field in items(declared_fields):
-            if name == FREETEXT_SEARCH_NAME:
-                continue
-            # We need to check if it's in declared_filters first, otherwise we remove any injected fields
-            if name in declared_filters and name not in self.filters:
-                set_and_remember_for_reinvoke(field, include=False)
-
         bind_members(self, name='endpoints')
 
         self.form = self.form.bind(parent=self)
@@ -1057,19 +1024,15 @@ class Query(Part):
         return self._lowercase_filters_cache
 
     @classmethod
-    @dispatch(
-        filters=EMPTY,
-    )
-    def filters_from_model(cls, filters, **kwargs):
+    @dispatch()
+    def filters_from_model(cls, **kwargs):
         return create_members_from_model(
-            member_class=cls.get_meta().member_class, member_params_by_member_name=filters, **kwargs
+            member_class=cls.get_meta().member_class, **kwargs
         )
 
     @classmethod
-    @dispatch(
-        filters=EMPTY,
-    )
-    def _from_model(cls, *, rows=None, model=None, filters, include=None, exclude=None):
+    @dispatch()
+    def _from_model(cls, *, rows=None, model=None, include=None, exclude=None):
         assert rows is None or isinstance(rows, QuerySet), (
             'auto__rows needs to be a QuerySet for filter generation to work. '
             'If it needs to be a lambda, provide a model with auto__model for filter generation, '
@@ -1078,5 +1041,5 @@ class Query(Part):
 
         model, rows = model_and_rows(model, rows)
         assert model is not None or rows is not None, "auto__model or auto__rows must be specified"
-        filters = cls.filters_from_model(model=model, include=include, exclude=exclude, filters=filters)
+        filters = cls.filters_from_model(model=model, include=include, exclude=exclude)
         return model, rows, filters
