@@ -2,7 +2,6 @@ from collections import defaultdict
 from contextlib import contextmanager
 from typing import (
     Any,
-    Dict,
     List,
     Type,
 )
@@ -10,31 +9,26 @@ from typing import (
 from tri_declarative import (
     dispatch,
     EMPTY,
-    flatten,
     get_shortcuts_by_name,
-    getattr_path,
     Namespace,
     RefinableObject,
     setdefaults_path,
 )
 
+from iommi._web_compat import settings
 from iommi.base import (
     items,
     keys,
 )
-from iommi.reinvokable import (
-    is_reinvokable,
-    retain_special_cases,
-)
-from ._web_compat import settings
+from iommi.refinable import RefinableObject
 
 DEFAULT_STYLE = 'bootstrap'
 
 
 def get_iommi_style_name(obj: Any) -> str:
     while obj is not None:
-        if obj.iommi_style is not None:
-            return obj.iommi_style
+        if obj.iommi_namespace.get('iommi_style') is not None:
+            return obj.iommi_namespace.iommi_style
         obj = obj.iommi_parent()
     return getattr(settings, 'IOMMI_DEFAULT_STYLE', DEFAULT_STYLE)
 
@@ -80,20 +74,6 @@ def get_style_object(obj: Any) -> 'Style':
     return resolved_stack[-1]
 
 
-def apply_style(style_object: 'Style', obj: Any, is_root) -> Any:
-    style_data = get_style_data_for_object(style_object, obj=obj, is_root=is_root)
-    return apply_style_data(style_data, obj)
-
-
-def apply_style_data(style_data: Namespace, obj: Any) -> Any:
-    if not style_data:
-        return obj
-    if not is_reinvokable(obj):
-        print(f'Missing out of {style_data} for {type(obj)}')
-        return obj
-    return reinvoke_new_defaults(obj, style_data)
-
-
 def _style_name_for_class(cls):
     return cls.__name__.rpartition('.')[-1]  # this converts iommi.form.Form to just Form
 
@@ -235,45 +215,6 @@ Available styles:
         ) from None
 
 
-def reinvoke_new_defaults(obj: Any, additional_kwargs: Dict[str, Any]) -> Any:
-    assert is_reinvokable(obj), (
-        f'reinvoke_new_defaults() called on object with ' f'missing @reinvokable constructor decorator: {obj!r}'
-    )
-    additional_kwargs_namespace = Namespace(additional_kwargs)
-
-    kwargs = Namespace(additional_kwargs_namespace)
-    for name, saved_param in items(obj._iommi_saved_params):
-        try:
-            new_param = getattr_path(additional_kwargs_namespace, name)
-        except AttributeError:
-            kwargs[name] = saved_param
-        else:
-            if is_reinvokable(saved_param):
-                assert isinstance(new_param, dict)
-                kwargs[name] = reinvoke_new_defaults(saved_param, new_param)
-            else:
-                if isinstance(saved_param, Namespace):
-                    kwargs[name] = Namespace(new_param, saved_param)
-                else:
-                    kwargs[name] = saved_param
-
-    try:
-        call_target = kwargs.pop('call_target', None)
-        if call_target is not None:
-            kwargs['call_target'] = Namespace(call_target, cls=type(obj))
-        else:
-            kwargs['call_target'] = type(obj)
-
-        result = kwargs()
-    except TypeError as e:
-        raise InvalidStyleConfigurationException(
-            f'Object {obj!r} could not be updated with style configuration {flatten(additional_kwargs_namespace)}'
-        ) from e
-
-    retain_special_cases(obj, result)
-    return result
-
-
 def get_style_data_for_object(style_object, obj, is_root):
     return style_object.component(obj, is_root)
 
@@ -319,6 +260,7 @@ def validate_styles(*, additional_classes: List[Type] = None, default_classes=No
         from iommi.fragment import Container
         from iommi.fragment import Header
         from iommi.live_edit import LiveEditPage
+        from iommi.form import FieldGroup
 
         default_classes = [
             Action,
@@ -339,6 +281,7 @@ def validate_styles(*, additional_classes: List[Type] = None, default_classes=No
             Container,
             Header,
             LiveEditPage,
+            FieldGroup,
         ]
     if additional_classes is None:
         additional_classes = []
