@@ -11,7 +11,6 @@ from enum import (
 from functools import total_ordering
 from io import StringIO
 from itertools import groupby
-from math import ceil
 from typing import (
     Any,
     Callable,
@@ -41,6 +40,7 @@ from django.utils.translation import (
     gettext,
     gettext_lazy,
 )
+from math import ceil
 from tri_declarative import (
     class_shortcut,
     declarative,
@@ -51,7 +51,6 @@ from tri_declarative import (
     Namespace,
     Refinable,
     refinable,
-    RefinableObject,
     setdefaults_path,
     Shortcut,
     with_meta,
@@ -84,7 +83,6 @@ from iommi.attrs import (
 )
 from iommi.base import (
     build_as_view_wrapper,
-    capitalize,
     get_display_name,
     items,
     keys,
@@ -119,7 +117,7 @@ from iommi.from_model import (
 )
 from iommi.member import (
     bind_members,
-    collect_members,
+    refine_done_members,
 )
 from iommi.page import (
     Page,
@@ -130,19 +128,16 @@ from iommi.query import (
     Q_OPERATOR_BY_QUERY_OPERATOR,
     Query,
 )
-from iommi.traversable import (
-    declared_members,
+from iommi.refinable import (
     evaluated_refinable,
     EvaluatedRefinable,
-    set_declared_member,
+    RefinableMembers,
+    RefinableObject,
+)
+from iommi.traversable import (
     Traversable,
 )
 from ._db_compat import base_defaults_factory
-from .reinvokable import (
-    reinvokable,
-    reinvoke,
-    set_and_remember_for_reinvoke,
-)
 
 LAST = LAST
 
@@ -363,14 +358,13 @@ class Column(Part):
     data_retrieval_method = EvaluatedRefinable()
     render_column: bool = EvaluatedRefinable()
 
-    @reinvokable
     @dispatch(
         attr=MISSING,
         sort_default_desc=False,
         sortable=lambda column, **_: column.attr is not None,
         auto_rowspan=False,
         bulk__include=False,
-        filter__include=False,
+        filter=EMPTY,
         data_retrieval_method=DataRetrievalMethods.attribute_access,
         cell__template=None,
         cell__attrs=EMPTY,
@@ -389,7 +383,7 @@ class Column(Part):
         header__url=None,
         render_column=True,
     )
-    def __init__(self, header, **kwargs):
+    def __init__(self, **kwargs):
         """
         Parameters with the prefix `filter__` will be passed along downstream to the `Filter` instance if applicable. This can be used to tweak the filtering of a column.
 
@@ -417,12 +411,14 @@ class Column(Part):
         model_field = kwargs.get('model_field')
         if model_field and model_field.remote_field:
             kwargs['model'] = model_field.remote_field.model
+        super(Column, self).__init__(**kwargs)
 
-        super(Column, self).__init__(header=HeaderColumnConfig(**header), **kwargs)
-
+    def on_refine_done(self):
+        self.header = HeaderColumnConfig(**self.header).refine_done(parent=self)
         self.is_sorting: bool = None
         self.sort_direction: str = None
         self.table = None
+        super(Column, self).on_refine_done()
 
     def __html__(self, *, render=None):
         assert (
@@ -563,6 +559,7 @@ class Column(Part):
         sortable=False,
         filter__is_valid_filter=lambda **_: (True, ''),
         filter__field__include=False,
+        attr=None,
     )
     def select(cls, checkbox_name='pk', checked=lambda row, **_: False, call_target=None, **kwargs):
         """
@@ -841,6 +838,7 @@ class Column(Part):
         return call_target(model_field=model_field, **kwargs)
 
 
+@with_meta
 class Cells(Traversable, Tag):
     """
     Internal class used in row rendering
@@ -853,9 +851,13 @@ class Cells(Traversable, Tag):
     extra_evaluated: Dict[
         str, Any
     ] = Refinable()  # not EvaluatedRefinable because this is an evaluated container so is special
+    cell_class: Type['Cell'] = Refinable()
 
-    def __init__(self, row, row_index, **kwargs):
-        super(Cells, self).__init__(_name='row', **kwargs)
+    def __init__(self, row, row_index, cell_class=None, **kwargs):
+        # This doesn't use a nice Meta definition because it would be a circular
+        if cell_class is None:
+            cell_class = Cell
+        super(Cells, self).__init__(_name='row', cell_class=cell_class, **kwargs)
         assert not isinstance(row, Cells)
         self.row: Any = row
         self.row_index = row_index
@@ -884,11 +886,11 @@ class Cells(Traversable, Tag):
         for column in values(self.iommi_parent().columns):
             if not column.render_column:
                 continue
-            yield Cell(cells=self, column=column)
+            yield self.cell_class(cells=self, column=column).refine_done(parent=self)
 
     def __getitem__(self, name):
         column = self.iommi_parent().columns[name]
-        return Cell(cells=self, column=column)
+        return self.cell_class(cells=self, column=column).refine_done(parent=self)
 
 
 class CellConfig(RefinableObject, Tag):
@@ -923,7 +925,8 @@ class Cell(CellConfig):
         self.table = cells.iommi_parent()
         self.row = cells.row
 
-        self._evaluate_parameters = {**self.cells.iommi_evaluate_parameters(), 'column': column}
+    def on_refine_done(self):
+        self._evaluate_parameters = {**self.cells.iommi_evaluate_parameters(), 'column': self.column}
 
         self.value = evaluate_strict(self.value, **self._evaluate_parameters)
         self._evaluate_parameters['value'] = self.value
@@ -1016,7 +1019,7 @@ class RowConfig(RefinableObject, Tag):
     extra_evaluated: Dict[str, Any] = Refinable()
 
     def as_dict(self):
-        return {k: getattr(self, k) for k in keys(self.get_declared('refinable_members'))}
+        return {k: getattr(self, k) for k in keys(self.get_declared('refinable'))}
 
 
 class ColumnHeader(object):
@@ -1025,7 +1028,9 @@ class ColumnHeader(object):
     read the docs for :doc:`HeaderConfig`.
     """
 
-    @dispatch()
+    @dispatch(
+        attrs=EMPTY,
+    )
     def __init__(
         self,
         *,
@@ -1205,12 +1210,14 @@ class Paginator(Traversable):
         ),
         slice=lambda top, bottom, rows, **_: rows[bottom:top],
     )
-    @reinvokable
     def __init__(self, **kwargs):
         super(Paginator, self).__init__(**kwargs)
+
+    def on_refine_done(self):
         self.context = None
         self.page_size = None
         self.rows = None
+        super(Paginator, self).on_refine_done()
 
     def on_bind(self) -> None:
         request = self.get_request()
@@ -1363,7 +1370,7 @@ def endpoint__csv(table, **_):
             return value
 
     def cell_value(cells, bound_column):
-        value = Cell(cells, bound_column).value
+        value = Cell(cells, bound_column).refine_done(parent=cells).value
         return bound_column.extra_evaluated.get('report_value', value)
 
     def rows():
@@ -1399,7 +1406,7 @@ class _Lazy_tbody:
         return mark_safe('\n'.join([cells.__html__() for cells in self.table.cells_for_rows()]))
 
 
-@declarative(Column, '_columns_dict')
+@declarative(Column, '_columns_dict', add_init_kwargs=False)
 @with_meta
 class Table(Part, Tag):
     """
@@ -1417,6 +1424,7 @@ class Table(Part, Tag):
 
     """
 
+    query = Refinable()
     bulk_filter: Namespace = EvaluatedRefinable()
     bulk_exclude: Namespace = EvaluatedRefinable()
     sortable: bool = EvaluatedRefinable()
@@ -1433,8 +1441,9 @@ class Table(Part, Tag):
     cell: CellConfig = EvaluatedRefinable()
     header = Refinable()
     model: Type[Model] = Refinable()  # model is evaluated, but in a special way so gets no EvaluatedRefinable type
-    initial_rows = Refinable()  # initial_rows is evaluated, but in a special way so gets no EvaluatedRefinable type
-    columns = Refinable()
+    rows = Refinable()  # rows is evaluated, but in a special way so gets no EvaluatedRefinable type
+    actions: Dict[str, Action] = RefinableMembers()
+    parts: Namespace = RefinableMembers()
     bulk: Optional[Form] = EvaluatedRefinable()
     bulk_container: Fragment = Refinable()
     superheader: Namespace = Refinable()
@@ -1451,9 +1460,14 @@ class Table(Part, Tag):
     query_class: Type[Query] = Refinable()
     action_class: Type[Action] = Refinable()
     page_class: Type[Page] = Refinable()
+    cells_class: Type[Cells] = Refinable()
 
     empty_message: str = Refinable()
     invalid_form_message: str = Refinable()
+    auto = Refinable()
+
+    # Columns need to be at the end to not steal the short names
+    columns: Dict[str, Column] = RefinableMembers()
 
     class Meta:
         assets__query_form_toggle_script__template = "iommi/query/form_toggle_script.html"
@@ -1463,6 +1477,7 @@ class Table(Part, Tag):
         query_class = Query
         action_class = Action
         page_class = Page
+        cells_class = Cells
         endpoints__tbody__func = lambda table, **_: {'html': table.__html__(template='iommi/table/table_container.html')}
         endpoints__csv__func = endpoint__csv
 
@@ -1493,7 +1508,6 @@ class Table(Part, Tag):
     def post_bulk_edit(table, queryset, updates, **_):
         pass
 
-    @reinvokable
     @dispatch(
         columns=EMPTY,
         bulk_filter={},
@@ -1539,22 +1553,11 @@ class Table(Part, Tag):
         # action button on the page. So let's use the secondary
         # style
         query__form__actions__submit__call_target=Action.button,
+        title=MISSING,
+        rows=None,
     )
     def __init__(
         self,
-        *,
-        columns: Namespace = None,
-        _columns_dict=None,
-        model=None,
-        rows=None,
-        bulk=None,
-        header=None,
-        query=None,
-        row=None,
-        parts: Namespace = None,
-        actions: Namespace = None,
-        auto,
-        title=MISSING,
         **kwargs,
     ):
         """
@@ -1567,27 +1570,30 @@ class Table(Part, Tag):
         :param bulk_exclude: exclude filters to apply to the `QuerySet` before performing the bulk operation
         :param sortable: set this to `False` to turn off sorting for all columns
         """
-        select_conf = columns.get('select', {})
-        if 'select' not in _columns_dict and isinstance(select_conf, dict):
-            columns['select'] = setdefaults_path(
-                Namespace(),
-                select_conf,
-                call_target__attribute='select',
-                attr=None,
-                after=-1,
-                include=False,
-            )
+        super(Table, self).__init__(**kwargs)
 
-        if auto:
-            auto = TableAutoConfig(**auto)
-            auto_model, auto_rows, columns = self._from_model(
+    def on_refine_done(self):
+        extra_column_defaults = {}
+        extra_column_defaults['select'] = dict(
+            call_target__attribute='select',
+            attr=None,
+            after=-1,
+            include=False,
+        )
+
+        model = self.model
+        rows = self.rows
+        if self.auto:
+            auto = TableAutoConfig(**self.auto).refine_done(parent=self)
+            auto_model, auto_rows, columns_from_auto = self._from_model(
                 model=auto.model,
                 rows=auto.rows,
-                columns=columns,
                 include=auto.include,
                 exclude=auto.exclude,
             )
 
+            assert 'select' not in columns_from_auto
+
             assert model is None, (
                 "You can't use the auto feature and explicitly pass model. "
                 "Either pass auto__model, or we will set the model for you from auto__rows"
@@ -1597,35 +1603,40 @@ class Table(Part, Tag):
             if rows is None:
                 rows = auto_rows
 
-            if title is MISSING:
-                title = f'{model._meta.verbose_name_plural.title()}'
+            if self.title is MISSING:
+                self.title = f'{model._meta.verbose_name_plural.title()}'
+        else:
+            columns_from_auto = None
 
-        if title is MISSING:
-            title = None
+        if self.title is MISSING:
+            self.title = None
 
-        model, rows = model_and_rows(model, rows)
+        self.model, self.rows = model_and_rows(model, rows)
 
-        assert isinstance(columns, dict)
-
-        self.columns = None
-
-        super(Table, self).__init__(
-            model=model, initial_rows=rows, header=HeaderConfig(**header), row=RowConfig(**row), title=title, **kwargs
-        )
+        self.initial_rows = self.rows
+        self.header = HeaderConfig(_name='header', **self.header).refine_done(parent=self)
+        self.row = RowConfig(**self.row).refine_done(parent=self)
 
         # In bind initial_rows will be used to set these 3 (in that order)
         self.sorted_rows = None
         self.sorted_and_filtered_rows = None
         self.visible_rows = None
 
-        collect_members(self, name='actions', items=actions, cls=self.get_meta().action_class)
-        collect_members(self, name='columns', items=columns, items_dict=_columns_dict, cls=self.get_meta().member_class)
-        collect_members(self, name='parts', items=parts, cls=Fragment)
+        refine_done_members(self, name='actions', members_from_namespace=self.actions, cls=self.get_meta().action_class, members_cls=Actions)
+        refine_done_members(self, name='columns', members_from_namespace=self.columns, members_from_declared=self.get_declared('_columns_dict'), members_from_auto=columns_from_auto, cls=self.get_meta().member_class, extra_member_defaults=extra_column_defaults,)
+
+        if not self.sortable:
+            for column in values(self.iommi_namespace.columns):
+                # Special case for entire table not sortable
+                column.sortable = False
+
+        refine_done_members(self, name='parts', members_from_namespace=self.parts, cls=Fragment)
+
+        query_args = self.query
+        bulk_args = self.bulk
 
-        self.query_args = query
         self.query: Query = None
-
-        self.bulk = None
+        self.bulk: Form = None
         self.header_levels = None
 
         def add_hidden_all_pks_field(declared_bulk_fields):
@@ -1644,9 +1655,11 @@ class Table(Part, Tag):
 
             field_class = self.get_meta().query_class.get_meta().member_class
 
-            for name, column in items(declared_members(self).columns):
-                # TODO: bulk does this, shouldn't this code also do it: `filter = query.fields.pop(name, {})` and then send that into the setdefaults_path below?
-
+            for name, column in items(self.iommi_namespace.columns):
+                if getattr(column, 'include', None) is False:
+                    continue
+                if getattr(column.filter, 'include', None) is False:
+                    continue
                 filter = setdefaults_path(
                     Namespace(),
                     column.filter,
@@ -1659,51 +1672,65 @@ class Table(Part, Tag):
                     field__display_name=column.display_name,
                 )
                 # Special case for automatic query config
-                if self.query_from_indexes and column.model_field and (getattr(column.model_field, 'db_index', False) or isinstance(column.model_field, AutoField)):
-                    filter.include = True
+                should_have_filter = bool(
+                    self.query_from_indexes
+                    and column.model_field
+                    and (getattr(column.model_field, 'db_index', False) or isinstance(column.model_field, AutoField))
+                )
+                setdefaults_path(
+                    filter,
+                    include=should_have_filter,
+                )
 
                 filters[name] = filter()
 
-            self.query = self.get_meta().query_class(
-                _filters_dict=filters, _name='query', model=self.model, **self.query_args
-            )
-            declared_members(self).query = self.query
+            self.query = self.get_meta().query_class(**setdefaults_path(
+                Namespace(),
+                query_args,
+                filters=filters,
+                _name='query',
+                model=self.model,
+            ))
+
+            declared_filters = self.query.iommi_namespace.filters
+            self.query = self.query.refine_defaults(filters=declared_filters)
 
             # Bulk
             field_class = self.get_meta().form_class.get_meta().member_class
 
             declared_bulk_fields = Struct()
-            for name, column in items(declared_members(self).columns):
-                field = bulk.fields.pop(name, {})
+            for name, column in items(self.iommi_namespace.columns):
+                if getattr(column, 'include', None) is False:
+                    continue
+                if getattr(column.bulk, 'include', None) is False:
+                    continue
+                field = setdefaults_path(
+                    Namespace(),
+                    column.bulk,
+                    dict(
+                        call_target__cls=field_class,
+                        model=self.model,
+                        model_field_name=column.model_field_name,
+                        _name=name,
+                        attr=name if column.attr is MISSING else column.attr,
+                        required=False,
+                        empty_choice_tuple=(None, '', '---', True),
+                        parse_empty_string_as_none=True,
+                        display_name=column.display_name,
+                    ),
+                )
+                if isinstance(column.model_field, BooleanField):
+                    field.call_target.attribute = 'boolean_tristate'
 
-                if column.bulk.include:
-                    field = setdefaults_path(
-                        Namespace(),
-                        column.bulk,
-                        dict(
-                            call_target__cls=field_class,
-                            model=self.model,
-                            model_field_name=column.model_field_name,
-                            _name=name,
-                            attr=name if column.attr is MISSING else column.attr,
-                            required=False,
-                            empty_choice_tuple=(None, '', '---', True),
-                            parse_empty_string_as_none=True,
-                            display_name=column.display_name,
-                        ),
-                        **field,
-                    )
-                    if isinstance(column.model_field, BooleanField):
-                        field.call_target.attribute = 'boolean_tristate'
-
-                    declared_bulk_fields[name] = field()
+                declared_bulk_fields[name] = field
 
             add_hidden_all_pks_field(declared_bulk_fields)
 
             # x.bulk.include can be a callable here. We treat that as truthy on purpose.
-            if any(x.bulk.include for x in values(declared_members(self).columns)) or 'actions' in bulk:
-                self.bulk = form_class(
-                    _fields_dict=declared_bulk_fields,
+            if any(x.bulk.include for x in values(self.iommi_namespace.columns)) or 'actions' in self.iommi_namespace.bulk:
+                self.bulk = form_class(**setdefaults_path(
+                    Namespace(),
+                    bulk_args,
                     _name='bulk',
                     model=self.model,
                     actions__submit=dict(
@@ -1717,31 +1744,35 @@ class Table(Part, Tag):
                         display_name=gettext_lazy('Bulk delete'),
                         include=False,
                     ),
-                    **bulk,
+                )).refine_defaults(
+                    fields=declared_bulk_fields,
                 )
 
-            declared_members(self).bulk = self.bulk
-
-        if not self.model and not self.bulk and 'actions' in bulk:
+        if not self.model and not self.bulk and 'actions' in self.iommi_namespace.bulk:  # @todo
             # Support custom 'bulk' actions even when there is no model
-            if any(x.bulk.include for x in values(declared_members(self).columns)):
+            if any(x.bulk.include for x in values(self.iommi_namespace.columns)):
                 assert False, "The builtin bulk actions only work on querysets."
             declared_bulk_fields = Struct()
             add_hidden_all_pks_field(declared_bulk_fields)
             self.bulk = form_class(
                 _name='bulk',
-                _fields_dict=declared_bulk_fields,
                 # We don't want form's default submit button unless somebody
                 # explicitly added it again.
-                actions__submit=bulk['actions'].get('submit', None),
-                **bulk,
+                actions__submit=bulk_args['actions'].get('submit', None),
+                **bulk_args,
+            ).refine_defaults(
+                fields=declared_bulk_fields,
             )
-            declared_members(self).bulk = self.bulk
 
-        # Columns need to be at the end to not steal the short names
-        declared_members(self).columns = declared_members(self).pop('columns')
+        if self.bulk is not None:
+            self.bulk = self.bulk.refine_done(parent=self)
 
-        self.bulk_container = self.bulk_container(_name='bulk_container')
+        if self.query is not None:
+            self.query = self.query.refine_done(parent=self)
+
+        self.bulk_container = self.bulk_container(_name='bulk_container').refine_done(parent=self)
+
+        super(Table, self).on_refine_done()
 
     @classmethod
     @class_shortcut(
@@ -1759,7 +1790,7 @@ class Table(Part, Tag):
         return self.parts.page
 
     def on_bind(self) -> None:
-        bind_members(self, name='actions', cls=Actions)
+        bind_members(self, name='actions')
         bind_members(self, name='columns')
         bind_members(self, name='endpoints')
         bind_members(self, name='parts')
@@ -1767,10 +1798,9 @@ class Table(Part, Tag):
         self.title = evaluate_strict(self.title, **self.iommi_evaluate_parameters())
         build_and_bind_h_tag(self)
 
-        self.tbody = self.tbody(_name='tbody').bind(parent=self)
+        self.tbody = self.tbody(_name='tbody', children__text=_Lazy_tbody(self)).bind(parent=self)
         self.container = self.container(_name='container').bind(parent=self)
         self.outer = self.outer(_name='outer').bind(parent=self)
-        self.tbody.children.text = _Lazy_tbody(self)
         self.header = self.header.bind(parent=self)
 
         # needs to be done first because _bind_headers depends on it
@@ -1786,30 +1816,20 @@ class Table(Part, Tag):
 
         self._prepare_sorting()
 
-        if not self.sortable:
-            # TODO: we could do this on the unbound stuff instead. This is bad because it triggers _bind_all()
-            for column in values(self.columns):
-                # Special case for entire table not sortable
-                column.sortable = False
-
-        # If the column is not included, the down stream query filters and bulk fields should also be gone
-        declared_query_filters = (
-            declared_members(self.query)['filters'] if self._declared_members.get('query') is not None else {}
-        )
-        declared_bulk_fields = (
-            declared_members(self.bulk)['fields'] if self._declared_members.get('bulk') is not None else {}
-        )
-        for name, column in items(self._declared_members.columns):
-            if name not in keys(self.columns):
-                if name in declared_query_filters:
-                    set_and_remember_for_reinvoke(declared_query_filters[name], include=False)
-                if name in declared_bulk_fields:
-                    set_and_remember_for_reinvoke(declared_bulk_fields[name], include=False)
-
         self._bind_query()
         self._bind_bulk_form()
         self._bind_headers()
 
+        # If the column is not included, the down stream query filters and bulk fields should also be gone
+        for name, column in items(self.iommi_namespace.columns):
+            if name not in keys(self.columns):
+                if self.query and name in self.query.filters:
+                    del self.query.filters[name]
+                    if self.query.form and name in self.query.form.fields:
+                        del self.query.form.fields[name]
+                if self.bulk and name in self.bulk.fields:
+                    del self.bulk.fields[name]
+
         if isinstance(self.sorted_and_filtered_rows, QuerySet):
             prefetch = [
                 x.attr
@@ -1844,15 +1864,6 @@ class Table(Part, Tag):
         if self.query is None:
             return
 
-        declared_filters = declared_members(self.query)['filters']
-        for name, column in items(declared_members(self)['columns']):
-            if name in declared_filters:
-                filter = Namespace(
-                    field__display_name=lambda table, field, **_: table.columns[field._name].display_name,
-                )
-                declared_filters[name] = reinvoke(declared_filters[name], filter)
-        set_declared_member(self.query, 'filters', declared_filters)
-
         self.query = self.query.bind(parent=self)
         self._bound_members.query = self.query
 
@@ -1869,17 +1880,6 @@ class Table(Part, Tag):
         if self.bulk is None:
             return
 
-        declared_fields = declared_members(self.bulk)['fields']
-        for name, column in items(declared_members(self)['columns']):
-            if name in declared_fields:
-                field = setdefaults_path(
-                    Namespace(),
-                    column.bulk,
-                    display_name=lambda table, field, **_: table.columns[field._name].display_name,
-                )
-                declared_fields[name] = reinvoke(declared_fields[name], field)
-        set_declared_member(self.bulk, 'fields', declared_fields)
-
         self.bulk = self.bulk.bind(parent=self)
         if self.bulk is not None:
             if self.bulk.actions:
@@ -1918,7 +1918,7 @@ class Table(Part, Tag):
                 prev_value = no_value_set
                 prev_row = no_value_set
                 for cells in self.cells_for_rows():
-                    value = Cell(cells, column).value
+                    value = Cell(cells, column).refine_done(parent=self).value
                     if prev_value != value:
                         rowspan_by_row[id(cells.row)] = 1
                         prev_value = value
@@ -2026,22 +2026,19 @@ class Table(Part, Tag):
         rows = self.preprocess_rows(rows=self.get_visible_rows(), **self.iommi_evaluate_parameters())
         for i, row in enumerate(rows):
             row = self.preprocess_row(table=self, row=row)
-            yield Cells(row=row, row_index=i, **self.row.as_dict()).bind(parent=self)
+            yield self.cells_class(row=row, row_index=i, **self.row.as_dict()).bind(parent=self)
 
     @classmethod
-    @dispatch(
-        columns=EMPTY,
-    )
-    def columns_from_model(cls, columns, **kwargs):
+    @dispatch()
+    def columns_from_model(cls, **kwargs):
         return create_members_from_model(
-            member_class=cls.get_meta().member_class, member_params_by_member_name=columns, **kwargs
+            member_class=cls.get_meta().member_class,
+            **kwargs,
         )
 
     @classmethod
-    @dispatch(
-        columns=EMPTY,
-    )
-    def _from_model(cls, *, rows=None, model=None, columns=None, include=None, exclude=None):
+    @dispatch()
+    def _from_model(cls, *, rows=None, model=None, include=None, exclude=None):
         assert rows is None or isinstance(rows, QuerySet), (
             'auto__rows needs to be a QuerySet for column generation to work. '
             'If it needs to be a lambda, provide a model with auto__model for column generation, '
@@ -2050,7 +2047,7 @@ class Table(Part, Tag):
 
         model, rows = model_and_rows(model, rows)
         assert model is not None or rows is not None, "auto__model or auto__rows must be specified"
-        columns = cls.columns_from_model(model=model, include=include, exclude=exclude, columns=columns)
+        columns = cls.columns_from_model(model=model, include=include, exclude=exclude)
         return model, rows, columns
 
     def _selection_identifiers(self):
